﻿using System;
using System.IO;
using System.Net.Sockets;
using System.Text;

using Vajhoej.Record;

namespace Vajhoej.VMS.TIG
{
	public class Connection
	{
		private TcpClient tc;
		private MemoryStream ms;
		private BinaryWriter bw;
		private BinaryReader br;
		public Connection(TcpClient tc)
		{
			this.tc = tc;
		}
		public void Prep()
		{
			ms = new MemoryStream();
			bw = new BinaryWriter(ms);
			bw.Write(0); // length to be updated later 
		}
		public void PutByte(byte v)
		{
			bw.Write(v);
		}
		public void PutShort(short v)
		{
			bw.Write(v);
		}
		public void PutInt(int v)
		{
			bw.Write(v);
		}
		public void PutLong(long v)
		{
			bw.Write(v);
		}
		public void PutFloat(float v)
		{
			bw.Write(v);
		}
		public void PutDouble(double v)
		{
			bw.Write(v);
		}
		public void PutString(string v)
		{
			byte[] temp = Encoding.GetEncoding(1252).GetBytes(v);
			bw.Write((short)temp.Length);
			bw.Write(temp);
		}
		public void PutBytes(byte[] v)
		{
			bw.Write((short)v.Length);
			bw.Write(v);
		}
		public void PutBlock(Object o)
		{
			StructWriter sw = new StructWriter();
			sw.Write(o);
			byte[] temp = sw.GetBytes();
			PutBytes(temp);
		}
		public void Send()
		{
			int wrlen = (int)ms.Length - 4;
			bw.Seek(0, SeekOrigin.Begin);
			bw.Write(wrlen);  // update length
			tc.GetStream().Write(ms.GetBuffer(), 0, wrlen + 4);
			tc.GetStream().Flush();
		}
		public void Recv()
		{
			br = new BinaryReader(tc.GetStream());
			int frlen = br.ReadInt32();
			byte[] fr = br.ReadBytes(frlen);
			ms = new MemoryStream(fr);
			br = new BinaryReader(ms);
		}
		public byte GetByte()
		{
			return br.ReadByte();
		}
		public short GetShort()
		{
			return br.ReadInt16();
		}
		public int GetInt()
		{
			return br.ReadInt32();
		}
		public long GetLong()
		{
			return br.ReadInt64();
		}
		public float GetFloat()
		{
			return br.ReadSingle();
		}
		public double GetDouble()
		{
			return br.ReadDouble();
		}
		public string GetString()
		{
			int len = br.ReadInt16();
			byte[] temp = br.ReadBytes(len);
			return Encoding.GetEncoding(1252).GetString(temp);
		}
		public byte[] GetBytes()
		{
			int len = br.ReadInt16();
			byte[] temp = br.ReadBytes(len);
			return temp;
		}
		public T GetBlock<T>(Type t) where T : class, new()
		{
		    byte[] temp = GetBytes();
			StructReader sr = new StructReader(temp);
			return sr.Read<T>(t);
		}
		public void GetBlock<T>(T o) where T : class, new()
		{
		    byte[] temp = GetBytes();
			StructReader sr = new StructReader(temp);
			sr.Read(o);
		}
		public void Close()
		{
			bw.Close();
			br.Close();
		}
	}
}